/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AQDB_OBJECT_P_H
#define AQDB_OBJECT_P_H

#include "aqdb_object.h"

/**
 * This is one of the entries of the field translation list.
 * For each field of an object there can be only one translation.
 * The purpose of these translations is to only have to make such a translation
 * once and store the result.
 * E.g. the first call to @ref AQDB_Object_GetFieldAsValue will translate the
 * string stored in the database to a AQDB_VALUE object and store that object in this
 * cache so that further calls to @ref AQDB_Object_GetFieldAsValue will just return that
 * value.
 */
typedef struct AQDB_OBJECT_FIELD AQDB_OBJECT_FIELD;
struct AQDB_OBJECT_FIELD {
  uint32_t fieldLen;
  uint32_t refCount;

  AQDB_DATA_TYPE type;
  void *typedPointer;

  uint8_t dataBegin[0];
};


typedef struct AQDB_OBJECT_SHARED AQDB_OBJECT_SHARED;
struct AQDB_OBJECT_SHARED {
  AQDB_ID tableId;
  AQDB_ID objectId;
  uint32_t fieldCount;

  uint32_t refCount;

  AQDB_OBJECT_FIELD *fieldPointers[0];
};



struct AQDB_OBJECT {
  GWEN_LIST_ELEMENT(AQDB_OBJECT)
  AQDB_OBJECT_SHARED *shared;

  uint32_t refCount;
};



static AQDB_OBJECT_FIELD *AQDB_ObjectField_new(const uint8_t *p, uint32_t len);
static void AQDB_ObjectField_Attach(AQDB_OBJECT_FIELD *field);
static void AQDB_ObjectField_free(AQDB_OBJECT_FIELD *field);
static void AQDB_ObjectField_ClearTypedPointer(AQDB_OBJECT_FIELD *field);

static AQDB_OBJECT_FIELD *AQDB_ObjectField_dup(const AQDB_OBJECT_FIELD *field);


static AQDB_OBJECT_SHARED *AQDB_ObjectShared_new(AQDB_ID tableId, AQDB_ID objectId, uint32_t nfields);
static void AQDB_ObjectShared_Attach(AQDB_OBJECT_SHARED *so);
static void AQDB_ObjectShared_free(AQDB_OBJECT_SHARED *so);
static AQDB_OBJECT_SHARED *AQDB_ObjectShared_dup(const AQDB_OBJECT_SHARED *so);
static AQDB_OBJECT_SHARED *AQDB_ObjectShared_deepCopy(const AQDB_OBJECT_SHARED *so);



#endif

