/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DB_CACHE_P_H
#define AQDB_DB_CACHE_P_H


#include "aqdb_cache.h"

#include <gwenhywfar/idmap.h>




typedef struct AQDB_CACHE_ENTRY AQDB_CACHE_ENTRY;
GWEN_LIST_FUNCTION_DEFS(AQDB_CACHE_ENTRY, AQDB_CacheEntry)
GWEN_IDMAP_FUNCTION_DEFS(AQDB_CACHE_ENTRY, AQDB_CacheEntry)

struct AQDB_CACHE_ENTRY {
  GWEN_LIST_ELEMENT(AQDB_CACHE_ENTRY)

  uint32_t accessCounter;

  AQDB_ID tableId;
  AQDB_ID objectId;
  AQDB_OBJECT *objectPointer;
  uint32_t objectSize;

  uint32_t refCount;
};


static AQDB_CACHE_ENTRY *AQDB_CacheEntry_new();
static void AQDB_CacheEntry_free(AQDB_CACHE_ENTRY *ce);




typedef struct AQDB_CACHE_TABLE AQDB_CACHE_TABLE;
GWEN_LIST_FUNCTION_DEFS(AQDB_CACHE_TABLE, AQDB_CacheTable)
GWEN_IDMAP_FUNCTION_DEFS(AQDB_CACHE_TABLE, AQDB_CacheTable)

struct AQDB_CACHE_TABLE {
  GWEN_LIST_ELEMENT(AQDB_CACHE_TABLE)

  AQDB_ID tableId;
  AQDB_CACHE_ENTRY_IDMAP *entryMap;

  uint32_t refCount;
};


static AQDB_CACHE_TABLE *AQDB_CacheTable_new();
static void AQDB_CacheTable_free(AQDB_CACHE_TABLE *ct);




struct AQDB_CACHE {
  GWEN_INHERIT_ELEMENT(AQDB_CACHE)
  GWEN_LIST_ELEMENT(AQDB_CACHE)

  uint32_t maxEntryCount;
  uint32_t maxTotalSize;


  uint32_t usedTotalSize;

  AQDB_CACHE_TABLE_IDMAP *tableMap;
  AQDB_CACHE_TABLE_LIST *tableList;

  AQDB_CACHE_ENTRY_LIST *entryList;

  uint32_t statMaxUsedTotalSize;
  uint32_t statMaxUsedEntries;
  uint32_t statHits;
  uint32_t statMisses;
  uint32_t statEvicted;

  uint32_t refCount;
};



static int AQDB_Cache_AddEntry(AQDB_CACHE *cc, AQDB_CACHE_ENTRY *ce);
static void AQDB_Cache_DelEntry(AQDB_CACHE *cc, AQDB_CACHE_ENTRY *ce);



#endif



