/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqDatabase (c) by 2020 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

/* included from "layer.c" */


int AQDB_IoLayerCached_BeginEdit(AQDB_IOLAYER *ioLayer, uint32_t editFlags, const char *user)
{
  AQDB_IOLAYER_CACHED *xio;
  AQDB_TABLE_LIST *dbTypeTables;
  int rv;

  assert(ioLayer);
  xio=GWEN_INHERIT_GETDATA(AQDB_IOLAYER, AQDB_IOLAYER_CACHED, ioLayer);
  assert(xio);

  if (xio->beginEditFn==NULL) {
    DBG_ERROR(AQDB_LOGDOMAIN, "Some virtual functions are not set in base layer");
    return GWEN_ERROR_NOT_IMPLEMENTED;
  }

  rv=xio->beginEditFn(ioLayer, editFlags, user);
  if (rv<0) {
    DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  dbTypeTables=AQDB_IoLayer_GetDbTypeTables(ioLayer);
  if (dbTypeTables) {
    AQDB_TABLE *dbt;

    dbt=AQDB_Table_List_First(dbTypeTables);
    while (dbt) {
      rv=AQDB_IoLayerCached_BeginEditTable(ioLayer, dbt);
      if (rv<0) {
        DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
        return rv;
      }
      dbt=AQDB_Table_List_Next(dbt);
    }
  }

  return 0;
}



int AQDB_IoLayerCached_EndEdit(AQDB_IOLAYER *ioLayer, uint32_t flags)
{
  AQDB_IOLAYER_CACHED *xio;
  AQDB_TABLE_LIST *dbTypeTables;
  int rv;

  assert(ioLayer);
  xio=GWEN_INHERIT_GETDATA(AQDB_IOLAYER, AQDB_IOLAYER_CACHED, ioLayer);
  assert(xio);

  if (xio->endEditFn==NULL) {
    DBG_ERROR(AQDB_LOGDOMAIN, "Some virtual functions are not set in base layer");
    return GWEN_ERROR_NOT_IMPLEMENTED;
  }

  dbTypeTables=AQDB_IoLayer_GetDbTypeTables(ioLayer);
  if (dbTypeTables) {
    AQDB_TABLE *dbt;

    dbt=AQDB_Table_List_First(dbTypeTables);
    while (dbt) {
      rv=AQDB_IoLayerCached_EndEditTable(ioLayer, dbt, flags);
      if (rv<0) {
        DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
        return rv;
      }
      dbt=AQDB_Table_List_Next(dbt);
    }
  }

  rv=xio->endEditFn(ioLayer, flags);
  if (rv<0) {
    DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



