/****************************************************************************
 * This file is part of the project AqFinance.
 * AqDataBase (c) by 2021 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "table_p.h"

#include <gwenhywfar/memory.h>
#include <gwenhywfar/endianfns.h>
#include <gwenhywfar/debug.h>





GWEN_INHERIT_FUNCTIONS(AQDB_TABLE)
GWEN_LIST_FUNCTIONS(AQDB_TABLE, AQDB_Table)



AQDB_TABLE *AQDB_Table_new()
{
  AQDB_TABLE *dbt;

  GWEN_NEW_OBJECT(AQDB_TABLE, dbt);
  GWEN_INHERIT_INIT(AQDB_TABLE, dbt);
  GWEN_LIST_INIT(AQDB_TABLE, dbt);

  dbt->columnList=AQDB_Column_List_new();

  return dbt;
}



void AQDB_Table_free(AQDB_TABLE *dbt)
{
  if (dbt) {
    GWEN_LIST_FINI(AQDB_TABLE, dbt);
    GWEN_INHERIT_FINI(AQDB_TABLE, dbt);
    free(dbt->name);

    AQDB_Column_List_free(dbt->columnList);

    GWEN_FREE_OBJECT(dbt);
  }
}



const char *AQDB_Table_GetName(const AQDB_TABLE *dbt)
{
  assert(dbt);
  return dbt->name;
}



void AQDB_Table_SetName(AQDB_TABLE *dbt, const char *s)
{
  assert(dbt);
  free(dbt->name);
  if (s)
    dbt->name=strdup(s);
  else
    dbt->name=NULL;
}



AQDB_ID AQDB_Table_GetTableId(const AQDB_TABLE *dbt)
{
  assert(dbt);
  return dbt->tableId;
}



void AQDB_Table_SetTableId(AQDB_TABLE *dbt, AQDB_ID i)
{
  assert(dbt);
  dbt->tableId=i;
}



uint32_t AQDB_Table_GetOpenFlags(const AQDB_TABLE *dbt)
{
  assert(dbt);
  return dbt->openFlags;
}



void AQDB_Table_SetOpenFlags(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->openFlags=i;
}



void AQDB_Table_AddOpenFlags(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->openFlags|=i;
}



void AQDB_Table_SubOpenFlags(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->openFlags&=~i;
}



uint32_t AQDB_Table_GetFlags(const AQDB_TABLE *dbt)
{
  assert(dbt);
  return dbt->flags;
}



void AQDB_Table_SetFlags(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->flags=i;
}



void AQDB_Table_AddFlags(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->flags|=i;
}



void AQDB_Table_SubFlags(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->flags&=~i;
}



AQDB_COLUMN_LIST *AQDB_Table_GetColumnList(const AQDB_TABLE *dbt)
{
  assert(dbt);
  return dbt->columnList;
}



void AQDB_Table_SetColumnList(AQDB_TABLE *dbt, AQDB_COLUMN_LIST *cl)
{
  assert(dbt);
  if (cl!=dbt->columnList)
    AQDB_Column_List_free(dbt->columnList);
  dbt->columnList=cl;
}



uint32_t AQDB_Table_GetMaxObjectSize(const AQDB_TABLE *dbt)
{
  assert(dbt);
  return dbt->maxObjectSize;
}



void AQDB_Table_SetMaxObjectSize(AQDB_TABLE *dbt, uint32_t i)
{
  assert(dbt);
  dbt->maxObjectSize=i;
}



int AQDB_Table_GetColumnCount(const AQDB_TABLE *dbt)
{
  assert(dbt);
  if (dbt->columnList)
    return AQDB_Column_List_GetCount(dbt->columnList);
  return 0;
}



