/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DBDIR_P_H
#define AQDB_DBDIR_P_H

#include "dbdir_l.h"
#include "table_l.h"

#include <aqdatabase/backendsupport/aqdb_sessionlog.h>

#include <gwenhywfar/fslock.h>




typedef struct AQDB_DB_DIR AQDB_DB_DIR;
struct AQDB_DB_DIR {
  uint32_t openFlags;
  char *dataFolder;
  AQDB_DBDIR_TABLE *tableList[AQDB_DB_DIR_MAX_OPEN_TABLES];
  GWEN_FSLOCK *dirLock;

  AQDB_DBDIR_TABLE *sessionTable;
  AQDB_DBDIR_TABLE *rollbackTable;

  AQDB_ID currentSessionId;
  char *currentUserName;
  AQDB_ID firstSessionObjectId;
  AQDB_ID lastSessionObjectId;
  uint32_t sessionFlags;

  uint32_t sessionAdditions;
  uint32_t sessionDeletions;
  uint32_t sessionModifications;

};

void GWENHYWFAR_CB AQDB_DbDir_FreeData(void *bp, void *p);


int GWENHYWFAR_CB AQDB_DbDir_Open(AQDB_DB *db, uint32_t flags);
int GWENHYWFAR_CB AQDB_DbDir_Create(AQDB_DB *db, uint32_t flags);
int GWENHYWFAR_CB AQDB_DbDir_Close(AQDB_DB *db, uint32_t flags);

int GWENHYWFAR_CB AQDB_DbDir_OpenTable(AQDB_DB *db,
                                       uint32_t aflags,
                                       const char *tname,
                                       uint32_t flags,
                                       AQDB_ID *pTableId);
AQDB_COLUMN_LIST *GWENHYWFAR_CB AQDB_DbDir_GetColumnList(AQDB_DB *db,
                                                         uint32_t aflags,
                                                         AQDB_ID tableId);
int GWENHYWFAR_CB AQDB_DbDir_CreateTable(AQDB_DB *db,
                                         const char *tname,
                                         uint32_t flags,
                                         const AQDB_COLUMN_LIST *columns);
int GWENHYWFAR_CB AQDB_DbDir_CloseTable(AQDB_DB *db,
                                        AQDB_ID tableId,
                                        uint32_t flags);
int GWENHYWFAR_CB AQDB_DbDir_GetTableNameList(AQDB_DB *db, GWEN_STRINGLIST *tableNameList);

int GWENHYWFAR_CB AQDB_DbDir_ReadObject(AQDB_DB *db,
                                        uint32_t aflags,
                                        AQDB_ID tableId,
                                        AQDB_ID objectId,
                                        AQDB_OBJECT **pObject);
int GWENHYWFAR_CB AQDB_DbDir_WriteObject(AQDB_DB *db,
                                         const AQDB_OBJECT *object);

int GWENHYWFAR_CB AQDB_DbDir_DeleteObject(AQDB_DB *db,
                                          AQDB_ID tableId,
                                          AQDB_ID objectId);
int GWENHYWFAR_CB AQDB_DbDir_AddObject(AQDB_DB *db,
                                       AQDB_ID tableId,
                                       AQDB_OBJECT *object);

int GWENHYWFAR_CB AQDB_DbDir_QuerySubmit(AQDB_DB *db,
                                         uint32_t aflags,
                                         AQDB_ID tableId,
                                         AQDB_ID refQueryId,
                                         const char *queryExpression,
                                         uint32_t flags,
                                         AQDB_ID *pQueryId);

int GWENHYWFAR_CB AQDB_DbDir_QueryGetFirst(AQDB_DB *db,
                                           AQDB_ID queryId,
                                           AQDB_ID *pIdListPtr,
                                           int *pIdListLen);

int GWENHYWFAR_CB AQDB_DbDir_QueryGetNext(AQDB_DB *db,
                                          AQDB_ID queryId,
                                          AQDB_ID *pIdListPtr,
                                          int *pIdListLen);

int GWENHYWFAR_CB AQDB_DbDir_QueryClose(AQDB_DB *db, AQDB_ID queryId);

int GWENHYWFAR_CB AQDB_DbDir_EndEdit(AQDB_DB *db, uint32_t flags);

int AQDB_DbDir_GetNextSessionId(AQDB_DB *db, AQDB_ID *pId);
int AQDB_DbDir_WriteSessionDescr(AQDB_DB *db, int t, AQDB_ID *pId);
int AQDB_DbDir_WriteSessionLog(AQDB_DB *db, int t,
                               AQDB_ID tableId,
                               AQDB_ID objectId,
                               const AQDB_OBJECT *object);

int AQDB_DbDir_Flush(AQDB_DB *db);

int AQDB_DbDir_WriteRollbackLog(AQDB_DB *db, const AQDB_SESSION_LOG *sl);
int AQDB_DbDir_Session2Rollback(AQDB_DB *db);
int AQDB_DbDir_CommitSession(AQDB_DB *db);

int AQDB_DbDir_EnsureSessionTables(AQDB_DB *db);

const char *AQDB_DbDir_GetCurrentUserName(const AQDB_DB *db);
void AQDB_DbDir_SetCurrentUserName(AQDB_DB *db, const char *s);

int AQDB_DbDir_EnsureRollbackTable(AQDB_DB *db);
int AQDB_DbDir_EnsureSessionTable(AQDB_DB *db);

void AQDB_DbDir_AllTablesEditEnd(AQDB_DB *db);


#endif
