/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DBDIR_TABLE_L_H
#define AQDB_DBDIR_TABLE_L_H

#include <gwenhywfar/list1.h>

typedef struct AQDB_DBDIR_TABLE AQDB_DBDIR_TABLE;
GWEN_LIST_FUNCTION_DEFS(AQDB_DBDIR_TABLE, AQDB_DbDir_Table)

#define AQDB_DBDIR_TABLE_FLAGS_VARIABLE_LENGTH       0x00000001


#include <aqdatabase/aqdb_db_be.h>

#include "object_descr_l.h"



AQDB_DBDIR_TABLE *AQDB_DbDir_Table_new(AQDB_DB *db,
                                       const char *tname,
                                       AQDB_ID id);

void AQDB_DbDir_Table_free(AQDB_DBDIR_TABLE *table);

const char *AQDB_DbDir_Table_GetName(const AQDB_DBDIR_TABLE *table);
AQDB_ID AQDB_DbDir_Table_GetId(const AQDB_DBDIR_TABLE *table);

int AQDB_DbDir_Table_IsOpen(const AQDB_DBDIR_TABLE *table);

int AQDB_DbDir_TableOpen(AQDB_DBDIR_TABLE *table,
                         uint32_t flags);

int AQDB_DbDir_TableCreate(AQDB_DBDIR_TABLE *table,
                           uint32_t flags,
                           const AQDB_COLUMN_LIST *columns);

int AQDB_DbDir_TableClose(AQDB_DBDIR_TABLE *table,
                          uint32_t flags);

int AQDB_DbDir_TableDelete(AQDB_DBDIR_TABLE *table);

int AQDB_DbDir_TableReset(AQDB_DBDIR_TABLE *table);


int AQDB_DbDir_TableReadObject(AQDB_DBDIR_TABLE *table,
                               AQDB_ID objectId,
                               AQDB_OBJECT **pObject);

int AQDB_DbDir_TableWriteObject(AQDB_DBDIR_TABLE *table,
                                const AQDB_OBJECT *object);

int AQDB_DbDir_TableDeleteObject(AQDB_DBDIR_TABLE *table,
                                 AQDB_ID id);

int AQDB_DbDir_TableAddObject(AQDB_DBDIR_TABLE *table,
                              AQDB_OBJECT *object);

/** basically the same as @ref AQDB_DbDir_TableAddObject(), but this function
 * saves the newly added object with the status "unused"
 */
int AQDB_DbDir_TableReserveObject(AQDB_DBDIR_TABLE *table, AQDB_OBJECT *object);


int AQDB_DbDir_TableQuerySubmit(AQDB_DBDIR_TABLE *table,
                                AQDB_ID refQueryId,
                                const char *queryExpression,
                                uint32_t flags,
                                AQDB_ID *pQueryId);

int AQDB_DbDir_TableQueryGetFirst(AQDB_DBDIR_TABLE *table,
                                  AQDB_ID queryId,
                                  AQDB_ID *pIdListPtr,
                                  int *pIdListLen);

int AQDB_DbDir_TableQueryGetNext(AQDB_DBDIR_TABLE *table,
                                 AQDB_ID queryId,
                                 AQDB_ID *pIdListPtr,
                                 int *pIdListLen);

int AQDB_DbDir_TableQueryClose(AQDB_DBDIR_TABLE *table, AQDB_ID queryId);


AQDB_COLUMN_LIST *AQDB_DbDir_TableGetColumnList(const AQDB_DBDIR_TABLE *table);
int AQDB_DbDir_TableGetColumnCount(const AQDB_DBDIR_TABLE *table);
AQDB_COLUMN *AQDB_DbDir_TableGetColumnByName(const AQDB_DBDIR_TABLE *table,
                                             const char *name,
                                             int *pIdx);
AQDB_COLUMN *AQDB_DbDir_TableGetColumnByIndex(const AQDB_DBDIR_TABLE *table, int idx);

int AQDB_DbDir_Table_Flush(AQDB_DBDIR_TABLE *table);


void AQDB_DbDir_Table_EditObjectAdded(AQDB_DBDIR_TABLE *table, AQDB_ID id);
int AQDB_DbDir_Table_EditIsObjectAdded(AQDB_DBDIR_TABLE *table, AQDB_ID id);

void AQDB_DbDir_Table_EditObjectDeleted(AQDB_DBDIR_TABLE *table, AQDB_ID id);
int AQDB_DbDir_Table_EditIsObjectDeleted(AQDB_DBDIR_TABLE *table, AQDB_ID id);

void AQDB_DbDir_Table_AddSessionObjectDescr(AQDB_DBDIR_TABLE *table, AQDB_DBDIR_OBJDESCR *od);
AQDB_ID AQDB_DbDir_Table_GetSessionObjectId(const AQDB_DBDIR_TABLE *table, AQDB_ID objId);

AQDB_DBDIR_OBJDESCR_LIST *AQDB_DbDir_Table_GetSessionObjectList(const AQDB_DBDIR_TABLE *table);

void AQDB_DbDir_Table_EditEnd(AQDB_DBDIR_TABLE *table);


#endif
