/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DBDIR_TABLE_P_H
#define AQDB_DBDIR_TABLE_P_H


#include "table_l.h"
#include <aqdatabase/backendsupport/aqdb_query_result.h>

#include <stdio.h>

#ifdef OS_WIN32
# include <windows.h>
#endif


#define AQDB_DBDIR_TABLE_VERSION_MAJOR 1
#define AQDB_DBDIR_TABLE_VERSION_MINOR 0
#define AQDB_DBDIR_TABLE_VERSION_MICRO 0

#define AQDB_DBDIR_TABLE_HEADER_SIZE                 256
#define AQDB_DBDIR_TABLE_HEADER_VERSION_MAJOR        0  /* 1 byte */
#define AQDB_DBDIR_TABLE_HEADER_VERSION_MINOR        1  /* 1 byte */
#define AQDB_DBDIR_TABLE_HEADER_VERSION_MICRO        2  /* 1 byte */
#define AQDB_DBDIR_TABLE_HEADER_RFU1                 3  /* 1 byte */
#define AQDB_DBDIR_TABLE_HEADER_FLAGS                4  /* 4 bytes */
#define AQDB_DBDIR_TABLE_HEADER_OBJECT_SIZE          8  /* 4 bytes */
#define AQDB_DBDIR_TABLE_HEADER_OBJECT_FIELDS        12 /* 4 bytes */
#define AQDB_DBDIR_TABLE_HEADER_HIGHEST_ID           16 /* 8 bytes */
#define AQDB_DBDIR_TABLE_HEADER_TYPES_START          24 /* 8 bytes */
#define AQDB_DBDIR_TABLE_HEADER_DATA_START           32 /* 8 bytes */
#define AQDB_DBDIR_TABLE_HEADER_MAX_OBJECTS          40 /* 8 bytes */


#define AQDB_DBDIR_TABLE_OBJECT_HEADER_SIZE          8
#define AQDB_DBDIR_TABLE_OBJECT_HEADER_STATUS_SIZE   4  /* size from begin of object header to (incl) status */
#define   AQDB_DBDIR_TABLE_OBJECT_HEADER_CHECKSUM    0
#define   AQDB_DBDIR_TABLE_OBJECT_HEADER_RFU1        1
#define   AQDB_DBDIR_TABLE_OBJECT_HEADER_RFU2        2
#define   AQDB_DBDIR_TABLE_OBJECT_HEADER_STATUS      3
#define   AQDB_DBDIR_TABLE_OBJECT_HEADER_LENGTH      4  /* 4 byte size */
#define   AQDB_DBDIR_TABLE_OBJECT_HEADER_DATA        8


#define AQDB_DBDIR_TABLE_OBJECT_STATUS_UNUSED        0
#define AQDB_DBDIR_TABLE_OBJECT_STATUS_ACTIVE        1
#define AQDB_DBDIR_TABLE_OBJECT_STATUS_DELETED       2


/* header for each column in an AQDB_OBJECT */
#define AQDB_DBDIR_TABLE_COLUMN_HEADER_SIZE          2

#define AQDB_DBDIR_MAX_QUERY 256



struct AQDB_DBDIR_TABLE {
  GWEN_LIST_ELEMENT(AQDB_DBDIR_TABLE)
  AQDB_DB *db;
  char *name;
  AQDB_ID tableId;
  char *dataFileName;

  uint32_t openFlags;

  uint8_t vMajor;
  uint8_t vMinor;
  uint8_t vMicro;
  AQDB_ID lastId;
  /** this normally is lastId+1, e.g. it is the highest number of objects ever
   * stored in the DB */
  AQDB_ID maxObjects;
  uint64_t typesOffset;
  uint64_t dataOffset;

  uint32_t flags;

  int objectSize;
  int objectColumns;

  AQDB_COLUMN_LIST *columnList;

  int modified;

#ifdef OS_WIN32
  HANDLE fh;
#else
  int fd;
#endif

  AQDB_QUERY_RESULT *queryResultList[AQDB_DBDIR_MAX_QUERY];

  GWEN_IDLIST64 *objectPositions;
  uint64_t fileSize;

  GWEN_IDLIST64 *editObjectsAdded;
  GWEN_IDLIST64 *editObjectsDeleted;

  AQDB_DBDIR_OBJDESCR_IDMAP *sessionObjectMap;
  AQDB_DBDIR_OBJDESCR_LIST *sessionObjectList;
};


int AQDB_DbDir_Table_ReadHeader(AQDB_DBDIR_TABLE *table);
int AQDB_DbDir_Table_WriteHeader(AQDB_DBDIR_TABLE *table);

int AQDB_DbDir_TableReadObjectAt(AQDB_DBDIR_TABLE *table,
                                 uint64_t filePos,
                                 AQDB_ID objectId,
                                 int maxLen,
                                 int maxColumns,
                                 AQDB_OBJECT **pObject);

int AQDB_DbDir_TableWriteObjectAt(AQDB_DBDIR_TABLE *table,
                                  uint64_t filePos,
                                  AQDB_ID objectId,
                                  int maxLen,
                                  int maxColumns,
                                  const AQDB_OBJECT *object,
                                  uint32_t status);

int AQDB_DbDir_Table_ColumnFromObject(AQDB_DBDIR_TABLE *table,
                                      const AQDB_OBJECT *o,
                                      AQDB_COLUMN **pCol);

int AQDB_DbDir_Table_ColumnToObject(AQDB_DBDIR_TABLE *table,
                                    const AQDB_COLUMN *col,
                                    AQDB_OBJECT *o);

int AQDB_DbDir_Table_WriteColumns(AQDB_DBDIR_TABLE *table);
int AQDB_DbDir_Table_ReadColumns(AQDB_DBDIR_TABLE *table);

int AQDB_DbDir__TableListObjects(AQDB_DBDIR_TABLE *table, GWEN_IDLIST64 *idList);
int AQDB_DbDir__TableListObjectsV(AQDB_DBDIR_TABLE *table, GWEN_IDLIST64 *idList);
int AQDB_DbDir__TableScanObjects(AQDB_DBDIR_TABLE *table, GWEN_IDLIST64 *posList);


#endif
