/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqFileStore (c) by 2020 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQFS_CHECK_H
#define AQFS_CHECK_H


/**
 * Checks the noted file size against the real file size. Repair if requested.
 *
 */
int AQFS_CheckFileSize(const char *fileName);


/**
 * Checks for every item in the given file whether it can be read.
 *
 * This is a very simple check. The items are simply read sequentially, the file structure is
 * not checked by this.
 */
int AQFS_CheckFileSequentially(const char *fileName, uint64_t *pNumTables, uint64_t *pNumItems);


/**
 * Check the structure of the given file.
 *
 * This check sequentially reads every item of the given file.
 * For every table item the table itself is read. If the next table pointer is set in the table item
 * that table is also read to check whether the pointer is also valid.
 * For every table entry pointer the object pointed to (if any) is also read. If that entry is also a pointer
 * to a table that table is handled recursively.
 * This effectively verifies the file structure on the file level.
 *
 */
int AQFS_CheckFileStructure(const char *fileName, uint64_t *pNumTables, uint64_t *pNumItems);



#endif /* AQFS_CHECK_H */
