/****************************************************************************
 * This file is part of the project AqFinance.
 * AqDataBase (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#if AQDATABASE_ENABLE_TESTCODE



/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */


static int GWENHYWFAR_CB test1(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test2(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test3a(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test3b(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test4(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test5(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test6(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test7(GWEN_TEST_MODULE *mod);
static int GWENHYWFAR_CB test8(GWEN_TEST_MODULE *mod);



/* ------------------------------------------------------------------------------------------------
 * implementations
 * ------------------------------------------------------------------------------------------------
 */




int AQFS_Table_PtrList_AddTests(GWEN_TEST_MODULE *mod)
{
  GWEN_TEST_MODULE *newMod;

  newMod=GWEN_Test_Module_AddModule(mod, "AFS_Table_PtrList", NULL);

  GWEN_Test_Module_AddTest(newMod, "test1", test1, NULL);
  GWEN_Test_Module_AddTest(newMod, "test2", test2, NULL);
  GWEN_Test_Module_AddTest(newMod, "test3a", test3a, NULL);
  GWEN_Test_Module_AddTest(newMod, "test3b", test3b, NULL);
  GWEN_Test_Module_AddTest(newMod, "test4", test4, NULL);
  GWEN_Test_Module_AddTest(newMod, "test5", test5, NULL);
  GWEN_Test_Module_AddTest(newMod, "test6", test6, NULL);
  GWEN_Test_Module_AddTest(newMod, "test7", test7, NULL);
  GWEN_Test_Module_AddTest(newMod, "test8", test8, NULL);

  return 0;
}



int test1(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<2048; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList1, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Error on AQFS_Table_PtrList_GetEntryAt(%d): %s (%d)", i, GWEN_Error_SimpleToString(entry),
                (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  return 0;
}



int test2(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  for (i=0; i<2048; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList1, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_GetEntryAt(%d, list1): %s (%d)",
                i, GWEN_Error_SimpleToString(entry), (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<2048; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList2, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_GetEntryAt(%d, list2): %s (%d)",
                i, GWEN_Error_SimpleToString(entry), (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  return 0;
}



int test3a(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }


  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table (list1) %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  AQFS_Table_PtrList_free(ptrList1);

  return 0;
}



int test3b(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }


  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table (list1) %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  DBG_ERROR(AQFS_LOGDOMAIN, "Breakpoint 1");

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  DBG_ERROR(AQFS_LOGDOMAIN, "Breakpoint 2");


  AQFS_Table_PtrList_free(ptrList1);

  DBG_ERROR(AQFS_LOGDOMAIN, "Breakpoint 3");

  AQFS_Table_PtrList_free(ptrList2);

  return 0;
}



int test4(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  for (i=0; i<2048; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList1, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_GetEntryAt(%d, list1): %s (%d)",
                i, GWEN_Error_SimpleToString(entry), (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<2048; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList2, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_GetEntryAt(%d, list2): %s (%d)",
                i, GWEN_Error_SimpleToString(entry), (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }


  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table (list1) %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table (list2) %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  AQFS_Table_PtrList_free(ptrList2);
  AQFS_Table_PtrList_free(ptrList1);

  return 0;
}



int test5(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  AQFS_Table_PtrList_free(ptrList1);

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table (list2) %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  AQFS_Table_PtrList_free(ptrList2);

  return 0;
}



int test6(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;
  int64_t iEntry;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table1;
    AQFS_TABLE *table2;

    table1=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    table2=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    if (table1!=table2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Table at position %d should be equal but isn't", i);
      return GWEN_ERROR_INTERNAL;
    }
  }


  iEntry=AQFS_Table_PtrList_AddEntry(ptrList2, (uint64_t) 2048);
  if (iEntry<0) {
    DBG_ERROR(AQFS_LOGDOMAIN,
              "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
              i, GWEN_Error_SimpleToString((int)iEntry), (int)iEntry);
    return (int)iEntry;
  }
  if (iEntry!=(int64_t)2048) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be 2048", i, (unsigned long) iEntry);
    return GWEN_ERROR_INTERNAL;
  }


  if (AQFS_Table_PtrList_GetEntryAt(ptrList1, 2048)>0) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Entry 2048 exists in list1 when it should not");
    return GWEN_ERROR_INTERNAL;
  }

  iEntry=AQFS_Table_PtrList_GetEntryAt(ptrList2, 2049);
  if (iEntry>0) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Entry 2049 exists in list2 when it should not (%ld)", (long int) iEntry);
    return GWEN_ERROR_INTERNAL;
  }


  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 2 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 2 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=8; i<9; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  return 0;
}



int test7(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2048; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  for (i=2048; i<4096; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList2, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }


  for (i=0; i<2048; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList1, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_GetEntryAt(%d, list1): %s (%d)",
                i, GWEN_Error_SimpleToString(entry), (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  if (AQFS_Table_PtrList_GetEntryAt(ptrList1, 2048)>0) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Entry 2048 exists in list1 when it should not");
    return GWEN_ERROR_INTERNAL;
  }

  for (i=0; i<4096; i++) {
    int64_t entry;

    entry=AQFS_Table_PtrList_GetEntryAt(ptrList2, i);
    if (entry<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_GetEntryAt(%d, list2): %s (%d)",
                i, GWEN_Error_SimpleToString(entry), (int)entry);
      return entry;
    }
    if (entry != (int64_t)(i+1)) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Non-matching entry at index %d: is %ld, should be %d ", i, (unsigned long) entry, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  if (AQFS_Table_PtrList_GetEntryAt(ptrList1, 4096)>=0) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Entry 4096 exists in list2 when it should not");
    return GWEN_ERROR_INTERNAL;
  }


  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 2 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 2 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=8; i<16; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }


  AQFS_Table_PtrList_free(ptrList1);

  for (i=0; i<(2048/256); i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table (list2) %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  return 0;
}



int test8(GWEN_UNUSED GWEN_TEST_MODULE *mod)
{
  AQFS_TABLE_PTRLIST *ptrList1;
  AQFS_TABLE_PTRLIST *ptrList2;
  int i;
  int rv;

  ptrList1=AQFS_Table_PtrList_new(256);

  for (i=0; i<2047; i++) {
    int64_t rv;

    rv=AQFS_Table_PtrList_AddEntry(ptrList1, (uint64_t) i+1);
    if (rv<0) {
      DBG_ERROR(AQFS_LOGDOMAIN,
                "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
                i, GWEN_Error_SimpleToString((int)rv), (int)rv);
      return rv;
    }
    if (rv!=(int64_t)i) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be %d ", i, (unsigned long) rv, i);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  ptrList2=AQFS_Table_PtrList_LazyCopy(ptrList1);

  DBG_ERROR(AQFS_LOGDOMAIN, "Breakpoint");

  rv=AQFS_Table_PtrList_AddEntry(ptrList2, (uint64_t) 2047);
  if (rv<0) {
    DBG_ERROR(AQFS_LOGDOMAIN,
              "Error on AQFS_Table_PtrList_AddEntry(%d): %s (%d)",
              i, GWEN_Error_SimpleToString((int)rv), (int)rv);
    return rv;
  }
  DBG_ERROR(AQFS_LOGDOMAIN, "Breakpoint");
  if (rv!=(int64_t)2047) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected index returned for entry %d: is %ld, should be 2047", i, (unsigned long) rv);
    return GWEN_ERROR_INTERNAL;
  }

  if (AQFS_Table_PtrList_GetTableAt(ptrList2, 8)) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Table 8 exists when it should not");
    return GWEN_ERROR_INTERNAL;
  }


  if (AQFS_Table_PtrList_GetEntryAt(ptrList2, 2049)>0) {
    DBG_ERROR(AQFS_LOGDOMAIN, "Entry 2049 exists in list2 when it should not");
    return GWEN_ERROR_INTERNAL;
  }


  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList1, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 2 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=0; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=2) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 2 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  for (i=8; i<8; i++) {
    AQFS_TABLE *table;
    int refCounter;

    table=AQFS_Table_PtrList_GetTableAt(ptrList2, i);
    refCounter=AQFS_Table_GetRefCounter(table);
    if (refCounter!=1) {
      DBG_ERROR(AQFS_LOGDOMAIN, "Unexpected reference counter in table %d: is %d, should be 1 ", i, refCounter);
      return GWEN_ERROR_INTERNAL;
    }
  }

  return 0;
}


#else

int AQFS_Table_PtrList_AddTests(GWEN_TEST_MODULE *mod)
{
  DBG_ERROR(AQFS_LOGDOMAIN, "AqDatabase was compiled without test code enabled.");
  return GWEN_ERROR_GENERIC;
}


#endif

