/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqFileStore (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/*
 * This tool scans a given AQFS file and checks whether it contains valid objects.
 */


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#undef BUILDING_AQDATABASE


#include <aqfilestorage/aqfs_file.h>
#include <aqfilestorage/aqfs_item_header.h>
#include <aqfilestorage/aqfs_check.h>


#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>

#include <unistd.h>



#define MAX_ITEM_LEN (1024*1024*10)

//#define DEBUG_OUTPUT


int main(int argc, char **argv)
{
  const char *fileName;
  int rv;
  uint64_t numOfTables=0;
  uint64_t numOfItems=0;
  int hasErrors=0;

  if (argc<2) {
    fprintf(stderr, "Usage: %s FILENAME\n", argv[0]);
    return 1;
  }

  fileName=argv[1];

  fprintf(stderr, "Checking file \"%s\"\n", fileName);

  /* first check */
  numOfTables=0;
  numOfItems=0;
  fprintf(stderr, "  1. Check: Checking readability of every item...\n");
  rv=AQFS_CheckFileSequentially(fileName, &numOfTables, &numOfItems);
  if (rv) {
    fprintf(stderr, "1. Check: Failure (%d)\n", rv);
    hasErrors++;
  }
  else {
    fprintf(stdout,
            "  1. Check: Success (read %llu tables, %llu items)\n",
            (unsigned long long int) numOfTables,
            (unsigned long long int) numOfItems);
  }

  /* second check */
  numOfTables=0;
  numOfItems=0;
  fprintf(stderr, "  2. Check: Checking file structure...\n");
  rv=AQFS_CheckFileStructure(fileName, &numOfTables, &numOfItems);
  if (rv) {
    fprintf(stderr, "2. Check: Failure (%d)\n", rv);
    hasErrors++;
  }
  else {
    fprintf(stdout,
            "  2. Check: Success (read %llu tables, %llu items)\n",
            (unsigned long long int) numOfTables,
            (unsigned long long int) numOfItems);
  }

  /* overall result */
  if (hasErrors)
    return 2;
  return 0;
}




