/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqFinance (c) by 2015 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DBFILE_P_H
#define AQDB_DBFILE_P_H


#include "dbfile_l.h"

#include <aqdatabase/aqdb_db.h>


#include <stdio.h>



#define AQDB_DBFILE_TYPETABLE_MAXENTRIES   128


#define AQDB_DBFILE_DBDATA_INDEX_HEADER 0
#define AQDB_DBFILE_DBDATA_INDEX_TYPES  1





typedef struct AQDB_DB_FILE AQDB_DB_FILE;
struct AQDB_DB_FILE {
  AQDB_IOLAYER *ioLayer;
  uint32_t openFlags;
  int isInEditMode;
};


static void GWENHYWFAR_CB AQDB_DbFile_FreeData(void *bp, void *p);



static int GWENHYWFAR_CB AQDB_DbFile_Create(AQDB_DB *db, uint32_t flags);

static int GWENHYWFAR_CB AQDB_DbFile_Open(AQDB_DB *db, uint32_t flags);

static int GWENHYWFAR_CB AQDB_DbFile_Close(AQDB_DB *db, uint32_t flags);

static int GWENHYWFAR_CB AQDB_DbFile_Check(AQDB_DB *db, GWEN_BUFFER *resultTextBuf);


static int GWENHYWFAR_CB AQDB_DbFile_CreateTable(AQDB_DB *db,
                                                 const char *name,
                                                 uint32_t flags,
                                                 const AQDB_COLUMN_LIST *columns);

static int GWENHYWFAR_CB AQDB_DbFile_OpenTable(AQDB_DB *db,
                                               uint32_t aflags,
                                               const char *tname,
                                               uint32_t flags,
                                               AQDB_ID *pTableId);

static int GWENHYWFAR_CB AQDB_DbFile_CloseTable(AQDB_DB *db,
                                                AQDB_ID tableId,
                                                uint32_t flags);

static int GWENHYWFAR_CB AQDB_DbFile_AddObject(AQDB_DB *db,
                                               AQDB_ID tableId,
                                               AQDB_OBJECT *object);

static int GWENHYWFAR_CB AQDB_DbFile_GetTableNameList(AQDB_DB *db, GWEN_STRINGLIST *tableNameList);


static int GWENHYWFAR_CB AQDB_DbFile_WriteObject(AQDB_DB *db, const AQDB_OBJECT *object);

static int GWENHYWFAR_CB AQDB_DbFile_ReadObject(AQDB_DB *db,
                                                uint32_t aflags,
                                                AQDB_ID tableId,
                                                AQDB_ID objectId,
                                                AQDB_OBJECT **pObject);

static int GWENHYWFAR_CB AQDB_DbFile_DeleteObject(AQDB_DB *db, AQDB_ID tableId, AQDB_ID objectId);


/**
 * Implementation of @ref AQDB_Db_BeginEdit.
 * @param db database
 * @param flags flags (see @ref AQDB_DBFILE_SESSION_FLAGS_ROLLBACK)
 * @param user name of the user starting the edit process.
 */
static int GWENHYWFAR_CB AQDB_DbFile_BeginEdit(AQDB_DB *db, uint32_t flags, const char *user);

static int GWENHYWFAR_CB AQDB_DbFile_EndEdit(AQDB_DB *db, uint32_t flags);

static int GWENHYWFAR_CB AQDB_DbFile_QuerySubmit(AQDB_DB *db,
                                                 uint32_t aflags,
                                                 AQDB_ID tableId,
                                                 AQDB_ID refQueryId,
                                                 const char *queryExpression,
                                                 uint32_t flags,
                                                 AQDB_ID *pQueryId);

static int GWENHYWFAR_CB AQDB_DbFile_QueryGetFirst(AQDB_DB *db,
                                                   AQDB_ID queryId,
                                                   AQDB_ID *pIdListPtr,
                                                   int *pIdListLen);

static int GWENHYWFAR_CB AQDB_DbFile_QueryGetNext(AQDB_DB *db,
                                                  AQDB_ID queryId,
                                                  AQDB_ID *pIdListPtr,
                                                  int *pIdListLen);

static int GWENHYWFAR_CB AQDB_DbFile_QueryClose(AQDB_DB *db, AQDB_ID queryId);


static AQDB_COLUMN_LIST GWENHYWFAR_CB *AQDB_DbFile_GetColumnList(AQDB_DB *db,
                                                                 uint32_t aflags,
                                                                 AQDB_ID tableId);

static int GWENHYWFAR_CB AQDB_DbFile_GetRollbackSessions(AQDB_DB *db,
                                                         uint32_t aflags,
                                                         AQDB_SESSION_DESCR_LIST *sl);

static int GWENHYWFAR_CB AQDB_DbFile_UndoSession(AQDB_DB *db, AQDB_ID sessionId);



#endif
