/****************************************************************************
 * This file is part of the project AqFinance.
 * AqDataBase (c) by 2016 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "header_p.h"
#include "dbfile_l.h"

#include <gwenhywfar/memory.h>
#include <gwenhywfar/endianfns.h>
#include <gwenhywfar/debug.h>




AQDB_DBFILE_HEADER *AQDB_DbFile_Header_new()
{
  AQDB_DBFILE_HEADER *dbh;

  GWEN_NEW_OBJECT(AQDB_DBFILE_HEADER, dbh);

  dbh->version=AQDB_DBFILE_VERSION;

  return dbh;
}



void AQDB_DbFile_Header_free(AQDB_DBFILE_HEADER *dbh)
{
  if (dbh) {
    GWEN_FREE_OBJECT(dbh);
  }
}




int AQDB_DbFile_Header_ReadFromBuffer(AQDB_DBFILE_HEADER *dbh, const uint8_t *buffer, uint64_t len)
{
  if (len<AQDB_DBFILE_HEADER_SIZE) {
    DBG_ERROR(AQDB_LOGDOMAIN, "Buffer too small for valid header data (%d < %d)",
              (int) len, (int) AQDB_DBFILE_HEADER_SIZE);
    return GWEN_ERROR_BAD_DATA;
  }

  dbh->version=GWEN_ENDIAN_LE64TOH(*((uint64_t *)(buffer+AQDB_DBFILE_HEADER_OFFS_VERSION)));
  dbh->lastSessionId=GWEN_ENDIAN_LE64TOH(*((uint64_t *)(buffer+AQDB_DBFILE_HEADER_OFFS_LAST_SESSIONID)));

  return 0;
}



AQDB_DBFILE_HEADER *AQDB_DbFile_Header_fromBuffer(const uint8_t *buffer, uint64_t len)
{
  AQDB_DBFILE_HEADER *dbh;
  int rv;

  dbh=AQDB_DbFile_Header_new();
  rv=AQDB_DbFile_Header_ReadFromBuffer(dbh, buffer, len);
  if (rv<0) {
    AQDB_DbFile_Header_free(dbh);
    return NULL;
  }

  return dbh;
}



int64_t AQDB_DbFile_Header_toBuffer(const AQDB_DBFILE_HEADER *dbh, uint8_t *buffer, uint64_t len)
{
  if (len<AQDB_DBFILE_HEADER_SIZE) {
    DBG_ERROR(AQDB_LOGDOMAIN, "Buffer too small for valid header data (%d < %d)",
              (int) len, (int) AQDB_DBFILE_HEADER_SIZE);
    return GWEN_ERROR_BAD_DATA;
  }
  memset(buffer, 0, AQDB_DBFILE_HEADER_SIZE);

  *((uint64_t *)(buffer+AQDB_DBFILE_HEADER_OFFS_VERSION))=GWEN_ENDIAN_HTOLE64(dbh->version);
  *((uint64_t *)(buffer+AQDB_DBFILE_HEADER_OFFS_LAST_SESSIONID))=GWEN_ENDIAN_HTOLE64(dbh->lastSessionId);

  return AQDB_DBFILE_HEADER_SIZE;
}



uint64_t AQDB_DbFile_Header_GetVersion(const AQDB_DBFILE_HEADER *dbh)
{
  assert(dbh);
  return dbh->version;
}



void AQDB_DbFile_Header_SetVersion(AQDB_DBFILE_HEADER *dbh, uint64_t i)
{
  assert(dbh);
  dbh->version=i;
}



uint64_t AQDB_DbFile_Header_GetLastSessionId(const AQDB_DBFILE_HEADER *dbh)
{
  assert(dbh);
  return dbh->lastSessionId;
}



void AQDB_DbFile_Header_SetLastSessionId(AQDB_DBFILE_HEADER *dbh, uint64_t i)
{
  assert(dbh);
  dbh->lastSessionId=i;
}








