/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqDatabase (c) by 2020 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "file/iolowlevel/io_query.h"
#include "file/iolowlevel/io_object.h"

#include <aqdatabase/aqdb_db.h>
#include <aqdatabase/aqdb_object_be.h>
#include <aqdatabase/aqdb_query.h>
#include <aqdatabase/backendsupport/aqdb_objectquery.h>
#include <aqdatabase/backendsupport/aqdb_query_result.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/endianfns.h>




/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */



/* ------------------------------------------------------------------------------------------------
 * implementations
 * ------------------------------------------------------------------------------------------------
 */



GWEN_IDLIST64 *AQDB_IoLayerLowlevel_GetActiveIds(AQDB_IOLAYER *ioLayer,
                                                 uint32_t accessFlags,
                                                 AQDB_TABLE *dbt,
                                                 uint32_t queryFlags)
{
  GWEN_IDLIST64 *idl;
  int isWriter;
  int rv;

  isWriter=(accessFlags & AQDB_ACCESS_FLAGS_ISWRITER);

  /* create a new id list */
  idl=GWEN_IdList64_newWithSteps(AQDB_DBFILE_IDLIST_ENTRIES);

  rv=AQDB_IoLayerLowlevel_Table_GetObjectIds(dbt, idl, isWriter);
  if (rv<0) {
    DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
    GWEN_IdList64_free(idl);
    return NULL;
  }

  if (GWEN_IdList64_GetEntryCount(idl)==0) {
    DBG_INFO(AQDB_LOGDOMAIN, "No entry in active list [%s]", AQDB_Table_GetName(dbt));
    GWEN_IdList64_free(idl);
    return NULL;
  }

  return idl;
}




