/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqDatabase (c) by 2020 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef AQDB_IOLAYER_LOWLEVEL_P_H
#define AQDB_IOLAYER_LOWLEVEL_P_H

#include "file/iolowlevel/layer.h"

#include <gwenhywfar/mutex.h>


typedef struct AQDB_IOLAYER_LOWLEVEL AQDB_IOLAYER_LOWLEVEL;
struct AQDB_IOLAYER_LOWLEVEL {
  char *dataFileName;
  AQFS_FILE *file;
  GWEN_FSLOCK *fileLock;

  /**
   * The first object in a database file is a table containing two entries for now:
   * - offset of the database header containing database information data
   * - offset of a AQFS_TABLE containing a list of type tables
   */
  AQFS_TABLE_LIST *tableListDatabase;

  /** Raw list of AQFS_TABLE objects containing the type information */
  AQFS_TABLE_LIST *tableListTypes;

  /* database header */
  AQDB_DBFILE_HEADER *header;

  AQDB_IOLAYER_CREATE_FN createFn;
  AQDB_IOLAYER_OPEN_FN openFn;
  AQDB_IOLAYER_CLOSE_FN closeFn;
  AQDB_IOLAYER_CHECK_FN checkFn;

  AQDB_IOLAYER_BEGINEDIT_FN beginEditFn;
  AQDB_IOLAYER_ENDEDIT_FN endEditFn;

  AQDB_IOLAYER_CREATETABLE_FN createTableFn;
  AQDB_IOLAYER_OPENTABLE_FN openTableFn;
  AQDB_IOLAYER_CLOSETABLE_FN closeTableFn;

  AQDB_IOLAYER_GETACTIVEIDSFORTABLE_FN getActiveIdsForTableFn;

  AQDB_IOLAYER_GETOBJECTPOS_FN getObjectPosFn;
  AQDB_IOLAYER_SETOBJECTPOS_FN setObjectPosFn;

  AQDB_IOLAYER_ADDOBJECT_FN addObjectFn;
  AQDB_IOLAYER_DELETEOBJECT_FN deleteObjectFn;
  AQDB_IOLAYER_WRITEOBJECT_FN writeObjectFn;
  AQDB_IOLAYER_READOBJECT_FN readObjectFn;

};



#endif

