/***************************************************************************
 begin       : Thu Oct 08 2020
 copyright   : (C) 2020 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "./dbopen.h"


#include <aqdatabase/aqdb.h>
#include <aqdatabase/aqdb_db.h>

#include <gwenhywfar/debug.h>




AQDB_DB *DbTool_OpenDatabase(const char *url, uint32_t oflags)
{
  int rv;
  AQDB_DB *db=NULL;

  rv=AQDB_DbFactory(url, &db);
  if (rv<0) {
    DBG_ERROR(0, "ERROR: Unable to instantiate db (%d)", rv);
    return NULL;
  }

  rv=AQDB_DB_Open(db, oflags);
  if (rv<0) {
    DBG_ERROR(0, "ERROR: Unable to open db (%d)", rv);
    AQDB_DB_free(db);
    return NULL;
  }

  return db;
}



AQDB_DB *DbTool_CreateDatabase(const char *url)
{
  int rv;
  AQDB_DB *db=NULL;

  rv=AQDB_DbFactory(url, &db);
  if (rv<0) {
    DBG_ERROR(0, "ERROR: Unable to instantiate db (%d)", rv);
    return NULL;
  }

  rv=AQDB_DB_Create(db, AQDB_ACTION_FLAGS_READ|AQDB_ACTION_FLAGS_WRITE);
  if (rv<0) {
    DBG_ERROR(0, "ERROR: Unable to create db (%d)", rv);
    AQDB_DB_free(db);
    return NULL;
  }

  rv=AQDB_DB_Open(db, AQDB_ACTION_FLAGS_READ|AQDB_ACTION_FLAGS_WRITE);
  if (rv<0) {
    DBG_ERROR(0, "ERROR: Unable to open db (%d)", rv);
    AQDB_DB_free(db);
    return NULL;
  }

  return db;
}





