/***************************************************************************
 begin       : Thu Oct 08 2020
 copyright   : (C) 2020 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "./listtables.h"
#include "./dbopen.h"

#include <aqdatabase/aqdb.h>
#include <aqdatabase/aqdb_db.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/args.h>



#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg




int DbTool_ListTables(GWEN_DB_NODE *dbArgs, int argc, char **argv)
{
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AQDB_DB *database;
  GWEN_STRINGLIST *tableNameList=NULL;
  const GWEN_ARGS args[]= {
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,           /* type */
      "dburl",                      /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "D",                          /* short option */
      "dburl",                      /* long option */
      I18S("Specify the database URL"),
      I18S("Specify the database URL")
    },
    {
      GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
      GWEN_ArgsType_Int,             /* type */
      "help",                       /* name */
      0,                            /* minnum */
      0,                            /* maxnum */
      "h",                          /* short option */
      "help",                       /* long option */
      I18S("Show this help screen"),    /* short description */
      I18S("Show this help screen")     /* long description */
    }
  };


  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);


  database=DbTool_OpenDatabase(dbUrl, AQDB_ACTION_FLAGS_READ);
  if (database==NULL) {
    DBG_ERROR(0, "Error opening database (%d)", rv);
    return 2;
  }

  tableNameList=GWEN_StringList_new();
  rv=AQDB_DB_GetTableNameList(database, tableNameList);
  if (rv<0) {
    DBG_ERROR(0, "Error getting table name list (%d)", rv);
    AQDB_DB_Close(database, AQDB_ACTION_FLAGS_ABORT);
    GWEN_StringList_free(tableNameList);
    AQDB_DB_free(database);
    return 3;
  }

  rv=AQDB_DB_Close(database, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error closing database (%d), ignoring", rv);
  }
  AQDB_DB_free(database);


  if (GWEN_StringList_Count(tableNameList)) {
    GWEN_STRINGLISTENTRY *se;

    se=GWEN_StringList_FirstEntry(tableNameList);
    while (se) {
      const char *s;

      s=GWEN_StringListEntry_Data(se);
      if (s && *s)
        fprintf(stdout, "%s\n", s);
      se=GWEN_StringListEntry_Next(se);
    }
  }
  GWEN_StringList_free(tableNameList);

  return 0;
}





